/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: BeanInfo.java 5602 2010-10-01 15:49:31Z sauthieg $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.container.info;

import org.ow2.easybeans.api.bean.info.IBeanInfo;
import org.ow2.easybeans.api.bean.info.ISecurityInfo;
import org.ow2.easybeans.api.bean.info.IWebServiceInfo;

import javax.ejb.ApplicationException;
import javax.ejb.TransactionManagementType;
import static javax.ejb.TransactionManagementType.CONTAINER;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * This class contains description for a bean.
 * It is used at the runtime.
 * @author Florent Benoit
 */
public class BeanInfo implements IBeanInfo {

    /**
     * Bean's name.
     */
    private String name;

    /**
     * Security info.
     */
    private ISecurityInfo securityInfo = null;

    /**
     * Management type for the bean.
     */
    private TransactionManagementType transactionManagementType = CONTAINER;

    /**
     * List of application exceptions used on this ejb-jar.
     */
    private Map<String, ApplicationException> applicationExceptions = null;

    /**
     * Cluster configuration.
     */
    private Object cluster;

    /**
     * Local interfaces.
     */
    private List<String> localInterfaces = null;

    /**
     * Remote interfaces.
     */
    private List<String> remoteInterfaces = null;

    /**
     * Web Services infos.
     */
    private IWebServiceInfo webserviceInfo;

    /**
     * Default constructor.
     */
    public BeanInfo() {
        this.localInterfaces = new ArrayList<String>();
        this.remoteInterfaces = new ArrayList<String>();
    }


    /**
     * Gets the list of application exceptions defined on this ejb jar metadata.
     * @return the list of application exceptions defined on this ejb jar metadata.
     */
    public Map<String, ApplicationException> getApplicationExceptions() {
        return this.applicationExceptions;
    }

    /**
     * Sets the list of application exceptions defined on this ejb jar metadata.
     * @param applicationExceptions the list of application exceptions defined on this ejb jar metadata.
     */
    public void setApplicationExceptions(final Map<String, ApplicationException> applicationExceptions) {
        this.applicationExceptions = applicationExceptions;
    }

    /**
     * Gets the type of transaction for the given bean.
     * @return transaction management type.
     */
    public TransactionManagementType getTransactionManagementType() {
        return this.transactionManagementType;
    }

    /**
     * Sets the type of transaction for the given bean.
     * @param transactionManagementType transaction management type.
     */
    public void setTransactionManagementType(final TransactionManagementType transactionManagementType) {
        this.transactionManagementType = transactionManagementType;
    }

    /**
     * Sets the security info.
     * @param securityInfo security info.
     */
    public void setSecurityInfo(final ISecurityInfo securityInfo) {
        this.securityInfo = securityInfo;
    }


    /**
     * Gets the security info.
     * @return security info.
     */
    public ISecurityInfo getSecurityInfo() {
        return this.securityInfo;
    }

    /**
     * Gets the name of the bean.
     * @return the name of the bean.
     */
    public String getName() {
        return this.name;
    }

    /**
     * Sets the name of the bean.
     * @param name the bean's name.
     */
    public void setName(final String name) {
        this.name = name;
    }

    /**
     * Gets configuration of the cluster, if the bean is clustered.
     * @return the cluster
     */
    public Object getCluster() {
        return this.cluster;
    }

    /**
     * Sets configuration of the cluster, if the bean is clustered.
     * @param cluster the cluster to set
     */
    public void setCluster(final Object cluster) {
        this.cluster = cluster;
    }

    /**
     * @return list of local interfaces (if any).
     */
    public List<String> getLocalInterfaces() {
        return this.localInterfaces;
    }

    /**
     * @param localInterfaces list of local interfaces.
     */
    public void setLocalInterfaces(final List<String> localInterfaces) {
        // ensure that the names are using ".'
        this.localInterfaces = fix(localInterfaces);
    }

    /**
     * @return list of remote interfaces (if any).
     */
    public List<String> getRemoteInterfaces() {
        return this.remoteInterfaces;
    }

    /**
     * @param remoteInterfaces list of remote interfaces.
     */
    public void setRemoteInterfaces(final List<String> remoteInterfaces) {
        // ensure that the names are using ".'
        this.remoteInterfaces = fix(remoteInterfaces);
    }

    /**
     * @return the webservices info related to this bean (if any)
     */
    public IWebServiceInfo getWebServiceInfo() {
        return this.webserviceInfo;
    }

    /**
     * @param info web services related runtime information.
     */
    public void setWebServiceInfo(IWebServiceInfo info) {
        this.webserviceInfo = info;
    }

    /**
     * Convert all / char into dot chars in the elements of the list.
     * @param list the parameter list
     * @return an updated list.
     */
    protected List<String> fix(final List<String> list) {
        if (list == null) {
            return null;
        }
        List<String> newList = new ArrayList<String>();
        for (String item : list) {
            newList.add(item.replace('/', '.'));
        }
        return newList;
    }

}
