/**
 * EasyBeans
 * Copyright (C) 2006-2007 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: MDBMessageEndPointFactory.java 5602 2010-10-01 15:49:31Z sauthieg $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.container.mdb;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

import javax.ejb.ActivationConfigProperty;
import javax.ejb.EJBException;
import javax.ejb.Timer;
import javax.resource.ResourceException;
import javax.resource.spi.ActivationSpec;
import javax.resource.spi.InvalidPropertyException;
import javax.resource.spi.ResourceAdapter;
import javax.resource.spi.UnavailableException;
import javax.resource.spi.endpoint.MessageEndpoint;
import javax.resource.spi.endpoint.MessageEndpointFactory;
import javax.transaction.xa.XAResource;

import org.ow2.easybeans.api.EZBContainer;
import org.ow2.easybeans.api.FactoryException;
import org.ow2.easybeans.api.bean.EasyBeansMDB;
import org.ow2.easybeans.resolver.api.EZBJNDIResolverException;
import org.ow2.util.ee.metadata.ejbjar.impl.struct.JActivationConfigProperty;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.pool.api.PoolException;

/**
 * Defines a class that will manage the message end point factory for the MDB.
 * The super class will manage the pool of message end point.
 * @author Florent Benoit
 */
public class MDBMessageEndPointFactory extends MDBFactory implements MessageEndpointFactory {

    /**
     * Default name of the activation spec (JORAM).
     */
    public static final String DEFAULT_ACTIVATION_SPEC_NAME = "joramActivationSpec";

    /**
     * Default name of the destination type.
     */
    public static final String DEFAULT_DESTINATION_TYPE = "javax.jms.Queue";

    /**
     * Activation Config Property for the destination.
     */
    public static final String DESTINATION_TYPE_PROPERTY = "destinationType";

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(MDBMessageEndPointFactory.class);

    /**
     * ActivationSpec object linked to this factory (used to activate or
     * deactive an endpoint factory (us).
     */
    private ActivationSpec activationSpec = null;

    /**
     * Resource adapter that provides the activation spec implementation.
     */
    private ResourceAdapter resourceAdapter = null;

    /**
     * Default constructor (delegate to super class).
     * @param className name of this factory (name of class that is managed)
     * @param container the root component of this factory.
     * @param activationSpec the activation Spec object used for
     *        activating/deactivating.
     * @param resourceAdapter the resource adapter used to activate/deactivate
     *        ourself.
     * @throws FactoryException if super constructor fails
     */
    public MDBMessageEndPointFactory(final String className, final EZBContainer container,
            final ActivationSpec activationSpec, final ResourceAdapter resourceAdapter) throws FactoryException {
        super(className, container);
        this.activationSpec = activationSpec;
        this.resourceAdapter = resourceAdapter;
    }

    /**
     * Init the factory.
     * @throws FactoryException if the initialization fails.
     */
    @Override
    public void init() throws FactoryException {
        initActivationSpec();

        validateActivationSpec();

        activate();
    }


    /**
     * Call setters method on the activation spec object.
     * @throws FactoryException if activation spec object is not configured.
     */
    private void initActivationSpec() throws FactoryException {

        // Get activation properties
        List<ActivationConfigProperty> properties = getMessageDrivenInfo().getActivationConfigProperties();
        // Init if null
        if (properties == null) {
            properties = new ArrayList<ActivationConfigProperty>();
        }

        // Message Destination Link to resolve ?
        String messageDestinationLink = getMessageDrivenInfo().getMessageDestinationLink();
        if (messageDestinationLink != null) {
            String jndiName = null;
            try {
                jndiName = getContainer().getConfiguration().getContainerJNDIResolver().getMessageDestinationJNDIUniqueName(
                        messageDestinationLink);
            } catch (EZBJNDIResolverException e) {
                throw new FactoryException("Unable to resolve message destination link '" + messageDestinationLink
                        + "' for bean '" + getBeanInfo().getName() + "'.", e);
            }
            properties.add(new JActivationConfigProperty("destination", jndiName));
            logger.info("Message destination link ''{0}'' resolved to ''{1}'' for bean ''{2}''", messageDestinationLink, jndiName, getBeanInfo().getName());
        }

            // Check that there is a destination-type, if not, add the default
            boolean destinationTypeFound = false;
            for (ActivationConfigProperty property : properties) {
                if (DESTINATION_TYPE_PROPERTY.equals(property.propertyName())) {
                    destinationTypeFound = true;
                    break;
                }
            }
            if (!destinationTypeFound) {
                ActivationConfigProperty jActivationConfigProperty = new JActivationConfigProperty(DESTINATION_TYPE_PROPERTY,
                        DEFAULT_DESTINATION_TYPE);
                properties.add(jActivationConfigProperty);
                logger.warn(
                        "No ''{0}'' property found in the activation config, adding default value ''{1}'' for bean ''{2}''",
                        DESTINATION_TYPE_PROPERTY, DEFAULT_DESTINATION_TYPE, getBeanInfo().getName());
            }


            // Call setter method for each property found in the activation
            // properties
            for (ActivationConfigProperty property : properties) {
                // get property name
                String key = property.propertyName();
                // and value
                String value = property.propertyValue();

                // define setter method name
                String methodName = "set" + key.substring(0, 1).toUpperCase() + key.substring(1);

                // get Method (reflection)
                Method m = null;
                try {
                    m = this.activationSpec.getClass().getMethod(methodName, new Class[] {String.class});
                } catch (SecurityException e) {
                    throw new FactoryException("Cannot get a method named '" + methodName
                            + "' on activation spec object '" + this.activationSpec + "'.", e);
                } catch (NoSuchMethodException e) {
                    throw new FactoryException("Cannot get a method named '" + methodName
                            + "' on activation spec object '" + this.activationSpec + "'.", e);
                }

                // invoke method
                try {
                    m.invoke(this.activationSpec, value);
                } catch (IllegalArgumentException e) {
                    throw new FactoryException("Cannot invoke method named '" + methodName + "' with value '" + value
                            + "' on activation spec object '" + this.activationSpec + "'.", e);
                } catch (IllegalAccessException e) {
                    throw new FactoryException("Cannot invoke method named '" + methodName + "' with value '" + value
                            + "' on activation spec object '" + this.activationSpec + "'.", e);
                } catch (InvocationTargetException e) {
                    throw new FactoryException("Cannot invoke method named '" + methodName + "' with value '" + value
                            + "' on activation spec object '" + this.activationSpec + "'.", e);
                }

            }


    }

    /**
     * Validate the configuration used p, tje activation spec object.
     * @throws FactoryException if the validation of the activation spec
     *         implementation object fails.
     */
    private void validateActivationSpec() throws FactoryException {
        try {
            this.activationSpec.validate();
        } catch (InvalidPropertyException e) {
            throw new FactoryException(
                    "Cannot validate the validation spec object for bean '" + getBeanInfo().getName() + "'.", e);
        }
    }

    /**
     * Activate this endpoint factory on resource adapter with the activation
     * spec object.
     * @throws FactoryException if the activation fails.
     */
    private void activate() throws FactoryException {
        try {
            this.resourceAdapter.endpointActivation(this, this.activationSpec);
        } catch (ResourceException e) {
            throw new FactoryException(
                    "Cannot activate the activationspec object and us (MessageEndPointFactory) on the resource adapter",
                    e);
        }
    }

    /**
     * This is used to create a message endpoint. The message endpoint is
     * expected to implement the correct message listener type.
     * @param xaResource an optional XAResource instance used to get transaction
     *        notifications when the message delivery is transacted.
     * @return a message endpoint instance.
     * @throws UnavailableException indicates a transient failure in creating a
     *         message endpoint. Subsequent attempts to create a message
     *         endpoint might succeed.
     */
    public MessageEndpoint createEndpoint(final XAResource xaResource) throws UnavailableException {
        // Use the internal method (which return a MDBMessageEndPoint object)
        return createInternalEndpoint(xaResource);
    }


    /**
     * This is used to create a message endpoint. The message endpoint is
     * expected to implement the correct message listener type.
     * @param xaResource an optional XAResource instance used to get transaction
     *        notifications when the message delivery is transacted.
     * @return a message endpoint instance.
     * @throws UnavailableException indicates a transient failure in creating a
     *         message endpoint. Subsequent attempts to create a message
     *         endpoint might succeed.
     */
    public MDBMessageEndPoint createInternalEndpoint(final XAResource xaResource) throws UnavailableException {
        // Instance that implements the MessageEndpoint interface
        MDBMessageEndPoint messageEndpoint = null;

        // get an instance of MDB
        EasyBeansMDB easyBeansMDB = null;
        try {
            easyBeansMDB = getPool().get();
        } catch (PoolException e) {
            throw new UnavailableException("Cannot get instance in the pool", e);
        }

        // Build a wrapper around this mdb instance
        // TODO: For now, this is only for JMS, needs to be changed.
        messageEndpoint = new MDBMessageListenerEndPoint(this, easyBeansMDB);

        // Set XAResource of the message endpoint.
        messageEndpoint.setXaResource(xaResource);

        return messageEndpoint;
    }


    /**
     * Release an endpoint created by this factory.
     * @param mdbMessageEndPoint the endpoint to release.
     */
    protected void releaseEndPoint(final MDBMessageEndPoint mdbMessageEndPoint) {
        // Release the wrapped message driven bean
        try {
            getPool().release(mdbMessageEndPoint.getEasyBeansMDB());
        } catch (PoolException e) {
            throw new IllegalStateException("Cannot release the given message end point", e);
        }
    }

    /**
     * This is used to find out whether message deliveries to a target method on
     * a message listener interface that is implemented by a message endpoint
     * will be transacted or not. The message endpoint may indicate its
     * transacted delivery preferences (at a per method level) through its
     * deployment descriptor. The message delivery preferences must not change
     * during the lifetime of a message endpoint.
     * @param method description of a target method. This information about the
     *        intended target method allows an application server to find out
     *        whether the target method call will be transacted or not.
     * @return boolean whether the specified method is transacted
     * @throws NoSuchMethodException exception to throw
     */
    public boolean isDeliveryTransacted(final Method method) throws NoSuchMethodException {
        // TODO : Not yet implemented
        return false;
    }

    /**
     * Stops the factory.
     */
    @Override
    public void stop() {
        // stop the pool.
        super.stop();

        // deactivate this factory
        this.resourceAdapter.endpointDeactivation(this, this.activationSpec);

    }




    /**
     * Notified when the timer service send a Timer object.
     * It has to call the Timed method.
     * @param timer the given timer object that will be given to the timer method.
     */
    public void notifyTimeout(final Timer timer) {
        // Get an EndPoint
        MDBMessageEndPoint mdbMessageEndPoint = null;
        try {
            mdbMessageEndPoint = createInternalEndpoint(null);
        } catch (UnavailableException e) {
            throw new EJBException("Cannot get an endpoint for notifying the timeout", e);
        }

        // Call the timeout method
        try {
            mdbMessageEndPoint.notifyTimeout(timer);
        } finally {
            // release the endpoint
            releaseEndPoint(mdbMessageEndPoint);
        }

    }


// /**
// * Notify a timeout for this bean
//         * @param timer timer whose expiration caused this notification.
//         */
//        public void notifyTimeout(Timer timer) {
//            if (stopped) {
//                TraceEjb.mdb.log(BasicLevel.WARN, "Container stopped");
//                return;
//            }
//            String methName = "notifyTimeout: ";
//            if (TraceEjb.isDebugJms()) {
//                TraceEjb.mdb.log(BasicLevel.DEBUG, methName);
//            }
//
//            // We need an instance from the pool to process the timeout.
//            JMessageEndpoint ep = null;
//            try {
//                ep = getNewInstance(null);
//            } catch (Exception e) {
//                TraceEjb.mdb.log(BasicLevel.ERROR, methName + "exception:" + e);
//                throw new EJBException("Cannot deliver the timeout", e);
//            }
//
//            // deliver the timeout to the bean
//            ep.deliverTimeout(timer);
//
//            // release the instance
//            releaseEndpoint(ep);
//        }
//    }
}
