/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: TomcatENCInterceptor.java 5602 2010-10-01 15:49:31Z sauthieg $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.naming.interceptors;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import javax.naming.Context;
import javax.naming.NamingException;

import org.ow2.easybeans.api.EasyBeansInvocationContext;
import org.ow2.easybeans.api.naming.NamingInterceptor;

/**
 * Interceptor used when EasyBeans is integrated in Tomcat. As the java:
 * namespace is managed by Tomcat, EasyBeans needs to call Tomcat objects in
 * order to set java: context.
 * @author Florent Benoit
 */
public class TomcatENCInterceptor implements NamingInterceptor {

    /**
     * Tomcat's class for naming.
     */
    protected static final String TOMCAT_NAMING_CLASS = "org.apache.naming.ContextBindings";

    /**
     * Method used to bind the name associated to a component context for the current thread on the Tomcat naming.
     */
    private Method bindThreadMethod = null;


    /**
     * Method used to get the context's name associated to the current thread on the Tomcat naming.
     */
    private Method getThreadNameMethod = null;

    /**
     * Method used to unbind the name associated to a context for the current thread on the Tomcat naming.
     */
    private Method unbindThreadMethod = null;

    /**
     * Method used to bind the component context for the current thread on the Tomcat naming.
     */
    private Method bindContextMethod = null;

    /**
     * Method used to unbind the component context for the current thread on the Tomcat naming.
     */
    private Method unbindContextMethod = null;


    /**
     * Default constructor. Gets a reference on the tomcat's naming methods
     */
    public TomcatENCInterceptor() {
        // Get reference on naming manager and its methods.
        if (this.bindThreadMethod == null) {
            String errMsg = "Check that EasyBeans is embedded in Tomcat web server.";
            Class namingClass = null;
            try {
                namingClass = Thread.currentThread().getContextClassLoader().loadClass(TOMCAT_NAMING_CLASS);
            } catch (ClassNotFoundException e) {
                throw new IllegalStateException("Cannot load the Tomcat naming class '"
                        + TOMCAT_NAMING_CLASS + "'. " + errMsg, e);
            }
            // get method (bindThread)
            try {
                this.bindThreadMethod = namingClass.getMethod("bindThread", new Class[] {Object.class});
            } catch (SecurityException e) {
                throw new IllegalStateException("Cannot get bindThread() method on the Tomcat naming class '"
                        + TOMCAT_NAMING_CLASS + "'. " + errMsg, e);
            } catch (NoSuchMethodException e) {
                throw new IllegalStateException("Cannot get bindThread() method on the Tomcat naming class '"
                        + TOMCAT_NAMING_CLASS + "'. " + errMsg, e);
            }

            // get method (getThreadName)
            try {
                this.getThreadNameMethod = namingClass.getDeclaredMethod("getThreadName", new Class[] {});
            } catch (SecurityException e) {
                throw new IllegalStateException("Cannot get getThreadName() method on the Tomcat naming class '"
                        + TOMCAT_NAMING_CLASS + "'. " + errMsg, e);
            } catch (NoSuchMethodException e) {
                throw new IllegalStateException("Cannot get getThreadName() method on the Tomcat naming class '"
                        + TOMCAT_NAMING_CLASS + "'. " + errMsg, e);
            }
            this.getThreadNameMethod.setAccessible(true);


            // get method (unbindThread)
            try {
                this.unbindThreadMethod = namingClass.getMethod("unbindThread", new Class[] {Object.class});
            } catch (SecurityException e) {
                throw new IllegalStateException("Cannot get unbindThread() method on the Tomcat naming class '"
                        + TOMCAT_NAMING_CLASS + "'. " + errMsg, e);
            } catch (NoSuchMethodException e) {
                throw new IllegalStateException("Cannot get unbindThread() method on the Tomcat naming class '"
                        + TOMCAT_NAMING_CLASS + "'. " + errMsg, e);
            }

            // get method (bindContext)
            try {
                this.bindContextMethod = namingClass.getMethod("bindContext", new Class[] {Object.class, Context.class});
            } catch (SecurityException e) {
                throw new IllegalStateException("Cannot get bindContext() method on the Tomcat naming class '"
                        + TOMCAT_NAMING_CLASS + "'. " + errMsg, e);
            } catch (NoSuchMethodException e) {
                throw new IllegalStateException("Cannot get bindContext() method on the Tomcat naming class '"
                        + TOMCAT_NAMING_CLASS + "'. " + errMsg, e);
            }

            // get method (unbindContext)
            try {
                this.unbindContextMethod = namingClass.getMethod("unbindContext", new Class[] {Object.class});
            } catch (SecurityException e) {
                throw new IllegalStateException("Cannot get unbindContext() method on the Tomcat naming class '"
                        + TOMCAT_NAMING_CLASS + "'. " + errMsg, e);
            } catch (NoSuchMethodException e) {
                throw new IllegalStateException("Cannot get unbindContext() method on the Tomcat naming class '"
                        + TOMCAT_NAMING_CLASS + "'. " + errMsg, e);
            }

        }
    }

    /**
     * Sets JOnAS ENC context.
     * @param invocationContext context with useful attributes on the current
     *        invocation.
     * @return result of the next invocation (to chain interceptors).
     * @throws Exception needs for signature of interceptor.
     */
    public Object intercept(final EasyBeansInvocationContext invocationContext) throws Exception {

        // Current context name associated to this thread
        Object name = null;
        try {
            name = this.getThreadNameMethod.invoke((Object[]) null);
        } catch (InvocationTargetException e) {
            // NamingException if no context is set on this thread
            if (!(e.getTargetException() instanceof NamingException)) {
                throw e;
            }
        }

        // Bind the new context name to the current thread
        this.bindThreadMethod.invoke(null, invocationContext.getFactory().getId());
        try {
            return invocationContext.proceed();
        } finally {
            // Reset
            if (name != null) {
                // bind with the previous context name (reset to previous value)
                this.bindThreadMethod.invoke(null, name);
            } else {
                // unbind value if no previous context name
                this.unbindThreadMethod.invoke(null, invocationContext.getFactory().getId());
            }
        }
    }

    /**
     * Init the context for the given name.
     * @param id the id.
     * @param context the context associated to a factory's id.
     */
    public void initContext(final String id, final Context context) {
        try {
            this.bindContextMethod.invoke(null, id, context);
        } catch (IllegalArgumentException e) {
            throw new IllegalStateException("Cannot init the context for the given id '" + id + "'.", e);
        } catch (IllegalAccessException e) {
            throw new IllegalStateException("Cannot init the context for the given id '" + id + "'.", e);
        } catch (InvocationTargetException e) {
            throw new IllegalStateException("Cannot init the context for the given id '" + id + "'.", e);
        }
    }

    /**
     * Remove the context associated to a given id.
     * @param id the id.
     */
    public void removeContext(final String id) {
        try {
            this.unbindContextMethod.invoke(null, id);
        } catch (IllegalArgumentException e) {
            throw new IllegalStateException("Cannot remove the context for the id '" + id + "'.", e);
        } catch (IllegalAccessException e) {
            throw new IllegalStateException("Cannot remove the context for the id '" + id + "'.", e);
        } catch (InvocationTargetException e) {
            throw new IllegalStateException("Cannot remove the context for the id '" + id + "'.", e);
        }
    }

}
